/***************************************************************************
                          itemdlg.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 2000 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <dlfcn.h>
#include <qfileinfo.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qlistview.h>
#include <qpopupmenu.h>
#include <klocale.h>
#include <kapp.h>
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  #include <kstddirs.h>
#endif
#include "itemdlg.h"
#include "kpldoc.h"
#include "kplchecklistitem.h"
#include "fitdlg.h"
#include "frameitem.h"
#include "arrayitem.h"
#include "funitem.h"
#include "legenditem.h"

#define ID_ACTIVE                   10010

NewItemDlg::NewItemDlg(QWidget *parent, int *i) :
 QDialog(parent, 0, true)
{
  ityp = i;
  setCaption(i18n("New item"));
  bg = new QButtonGroup();
  QRadioButton *r = groupButton(i18n("Frame"), 14, 22, 160, 24);
  r->setChecked(true);
  r = groupButton(i18n("Array"), 14, 54, 160, 24);
  r = groupButton(i18n("Function"), 14, 86, 160, 24);
  r = groupButton(i18n("Parametric Function"), 14, 118, 160, 24);
  r = groupButton(i18n("Legend"), 14, 150, 160, 24);
  QPushButton *b = new QPushButton(i18n("OK"), this);
  b->setGeometry(48, 192, 100, 25);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  adjustSize();
  setFixedSize(size());
}

NewItemDlg::~NewItemDlg()
{
}

QRadioButton *NewItemDlg::groupButton(const char *text,
                                      int x, int y, int w, int h)
{
  QRadioButton *b = new QRadioButton(text, this);
  b->setGeometry(x, y, w, h);
  bg->insert(b);
  return b;
}

void NewItemDlg::slotOK()
{
  QRadioButton *rb;

  *ityp = 0;
  do 
    rb = (QRadioButton *) bg->find((*ityp)++);
  while (!rb->isChecked());
  accept();
}

ItemDlg::ItemDlg(KplDoc *model)
{
  m = model;
  iarr = ifun = -1;
  popupMenu = new QPopupMenu();
  popupMenu->setCheckable(true);
  popupMenu->insertItem(i18n("&Edit"), this, SLOT(slotEdit()));
  popupMenu->insertItem(i18n("&Delete"), this, SLOT(slotDelete()));
  popupMenu->insertItem(i18n("&Copy"), this, SLOT(slotCopy()));
  popupMenu->insertItem(i18n("&Active"), this, SLOT(slotToggleState()), 0,
                        ID_ACTIVE);
  itemList = new QListView(this);
  itemList->setGeometry(42, 12, 378, 241);
  itemList->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  itemList->addColumn(i18n("Active"));
  itemList->addColumn(i18n("Type"));
  itemList->addColumn(i18n("Settings"));
  itemList->setRootIsDecorated(true);
  itemList->setAllColumnsShowFocus(true);
  itemList->setSorting(-1);
  connect(itemList, SIGNAL(returnPressed(QListViewItem *)),
          SLOT(slotList(QListViewItem *)));
  connect(itemList, SIGNAL(currentChanged(QListViewItem *)),
          SLOT(slotSelectionChanged(QListViewItem *)));
  connect(itemList, SIGNAL(rightButtonClicked(QListViewItem *, const QPoint &, int)),
          SLOT(slotPopup(QListViewItem *, const QPoint &, int)));
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
        KStandardDirs dirs;
	QPixmap pm(dirs.findResource("toolbar", "up.xpm"));
#else
	QPixmap pm(KApplication::kde_toolbardir() + "/up.xpm");
#endif
  up = new QPushButton(this);
  up->setGeometry(8, 96, 24, 24);
  up->setPixmap(pm);
  connect(up, SIGNAL(clicked()), SLOT(slotMoveUp()));
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  pm.load(dirs.findResource("toolbar", "down.xpm"));
#else
  pm.load(KApplication::kde_toolbardir() + "/down.xpm");
#endif
  down = new QPushButton(this);
  down->setGeometry(8, 130, 24, 24);
  down->setPixmap(pm);
  connect(down, SIGNAL(clicked()), SLOT(slotMoveDown()));
  QPushButton *b = new QPushButton(i18n("New"), this);
  b->setGeometry(430, 12, 90, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotNew()));
  edit = new QPushButton(i18n("Edit"), this);
  edit->setGeometry(430, 48, 90, 25);
  connect(edit, SIGNAL(clicked()), SLOT(slotEdit()));
  fit = new QPushButton(i18n("Fit"), this);
  fit->setGeometry(430, 84, 90, 25);
  connect(fit, SIGNAL(clicked()), SLOT(slotFit()));
  del = new QPushButton(i18n("Delete"), this);
  del->setGeometry(430, 120, 90, 25);
  connect(del, SIGNAL(clicked()), SLOT(slotDelete()));
  copy = new QPushButton(i18n("Copy"), this);
  copy->setGeometry(430, 156, 90, 25);
  connect(copy, SIGNAL(clicked()), SLOT(slotCopy()));
  b = new QPushButton(i18n("Help"), this);
  b->setGeometry(430, 192, 90, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotHelp()));
  b = new QPushButton(i18n("Close"), this);
  b->setGeometry(430, 228, 90, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotClose()));
  adjustSize();
  setFixedSize(size());
  updItemList();
  connect(this, SIGNAL(modelChanged()), m, SIGNAL(modelChanged()));
}

ItemDlg::~ItemDlg()
{
  delete popupMenu;
}

void ItemDlg::updItemList()
{
  KplCheckListItem *parent = 0;
  bool funcs = false, arrays = false;
  int current = -1;

  setCaption(m->currentFile() + " - " + i18n("Items"));
  if (!itemList->isMultiSelection())
    current = lItem.findRef((KplCheckListItem *) itemList->currentItem());
  itemList->clear();
  up->setEnabled(false);
  down->setEnabled(false);
  lItem.clear();
  int nItems = m->itd.count();
  if (nItems) {
    for (KplItem *item = m->itd.first(); item; item = m->itd.next()) {
      if (parent)
        if (item->iType() == FRAMEITEM)
          lItem.append(new KplCheckListItem(itemList, parent));
        else
          lItem.append(new KplCheckListItem(parent, lItem.getLast()));
      else
        if (item == m->itd.getFirst())
          lItem.append(new KplCheckListItem(itemList, lItem.getLast()));
        else
          lItem.append(new KplCheckListItem(itemList));
      if (item->iType() == FRAMEITEM) {
        parent = lItem.current();
        parent->setOpen(true);
      }
      item->setText(lItem.current(), &arrays, &funcs);
      lItem.current()->setOn(item->active);
      connect(lItem.current(), SIGNAL(stateChanged(KplCheckListItem *, bool)),
              SLOT(slotStateChanged(KplCheckListItem *, bool)));
    }
    KplCheckListItem *itm = lItem.at(QMAX(0, QMIN(current, nItems - 1)));
    itemList->setSelected(itm, true);
    itemList->setCurrentItem(itm);
    itemList->ensureItemVisible(itm);
  }
  edit->setEnabled(nItems);
  fit->setEnabled(arrays && funcs);
  copy->setEnabled(nItems);
  del->setEnabled(nItems);
  itemList->setFocus();
}

void ItemDlg::moveItem(bool upwards)
{
  int hi, lo, dest;

  if (upwards) {
    hi = lItem.findRef((KplCheckListItem *) itemList->currentItem());
    lo = dest = hi - 1;
  } else {
    lo = lItem.findRef((KplCheckListItem *) itemList->currentItem());
    hi = dest = lo + 1;
  }
  KplItem *hiItem = m->itd.at(hi);
  KplItem *loItem = m->itd.at(lo);
  m->itd.setAutoDelete(false);
  m->itd.remove(hi);
  m->itd.remove(lo);
  m->itd.insert(lo, hiItem);
  m->itd.insert(hi, loItem);
  m->itd.setAutoDelete(true);
  updItemList();
  itemList->setSelected(lItem.at(dest), true);
  m->setModified();
  emit modelChanged();
}

void ItemDlg::slotList(QListViewItem *it)
{
  int i = lItem.findRef((KplCheckListItem *) it);
  m->itd.at(i)->editItem(this, m, i);
  updItemList();
}

void ItemDlg::slotNew()
{
  int ityp;

  NewItemDlg dlg(this, &ityp);
  dlg.exec();
  m->iniStru((itemTypes) ityp);
  m->itd.last()->editItem(this, m, m->itd.count() - 1);
  updItemList();
}

void ItemDlg::slotEdit()
{
  QListViewItem *it;

  if (!itemList->isMultiSelection()) {
    if ((it = itemList->currentItem()))
      slotList(it);
  } else {
    itemList->setMultiSelection(false);
    updItemList();
  }
}

void ItemDlg::slotFit()
{
  uint i;

  if (itemList->isMultiSelection()) {
    int n = 0;
    for (i = 0; i < m->itd.count(); i++)
      if (itemList->isSelected(lItem.at(i))) {
        if (m->itd.at(i)->iType() == ARRAYITEM)
          iarr = i;
        if (m->itd.at(i)->iType() == FUNITEM)
          ifun = i;
        n++;
      }
    if ((n != 2) || (ifun < 0) || (iarr < 0))
      return;
  } else {
    int iarr0 = -1, ifun0 = -1, n = 0;
    for (i = 0; i < m->itd.count(); i++) {
      if (m->itd.at(i)->iType() == ARRAYITEM) {
        iarr0 = i;
        n++;
      }
      if (m->itd.at(i)->iType() == FUNITEM) {
        ifun0 = i;
        n++;
      }
    }
    if ((n != 2) || (ifun0 < 0) || (iarr0 < 0)) {

      itemList->setMultiSelection(true);
      if ((iarr >= 0) && (iarr < (int) m->itd.count()) &&
          (ifun >= 0) && (ifun < (int) m->itd.count())) {
        itemList->setSelected(lItem.at(iarr), true);
        itemList->setSelected(lItem.at(ifun), true);
      }
      emit displayMessage(i18n("Select one data set and one function!"));
      ((QWidget *) m->parent())->raise();
      itemList->setFocus();
      return;

    } else {
      iarr = iarr0;
      ifun = ifun0;
    }
  }
  emit displayMessage(i18n("Fitting function parameters..."));
  FitDlg dlg(this, m, (ArrayItem *) m->itd.at(iarr),
             (FunItem *) m->itd.at(ifun), SHOWDLG);
  connect(&dlg, SIGNAL(modelChanged()), m, SIGNAL(modelChanged()));
  dlg.exec();
  itemList->setMultiSelection(false);
  updItemList();
  emit displayMessage(i18n("Ready."));
}

void ItemDlg::slotDelete()
{
  if (!itemList->isMultiSelection()) {
    if (itemList->currentItem()) {
      int i = lItem.findRef((KplCheckListItem *) itemList->currentItem());
      m->itd.at(i)->freeItd(m);
      m->itd.remove(i);
      m->setModified();
      emit modelChanged();
    }
  } else
    itemList->setMultiSelection(false);
  updItemList();
}

void ItemDlg::slotCopy()
{
  if (!itemList->isMultiSelection()) {
    if (itemList->currentItem()) {
      m->itd.append(m->itd.at(lItem.findRef((KplCheckListItem *) itemList->currentItem()))->copy());
      m->setModified();
      emit modelChanged();
    }
  } else
    itemList->setMultiSelection(false);
  updItemList();
}

void ItemDlg::slotClose()
{
  hide();
}

void ItemDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-items");
}

void ItemDlg::slotPopup(QListViewItem *item, const QPoint &p, int)
{
  if (item) {
    itemList->setSelected(item, true);
    popupMenu->setItemChecked(ID_ACTIVE, ((KplCheckListItem *) item)->isOn());
    popupMenu->exec(p);
  }
}

void ItemDlg::slotStateChanged(KplCheckListItem *it, bool bState)
{
  m->itd.at(lItem.findRef(it))->active = bState;
  m->setModified();
  emit modelChanged();
}

void ItemDlg::slotToggleState()
{
  KplCheckListItem *item = (KplCheckListItem *) itemList->currentItem();
  if (item)
    item->setOn(!item->isOn());
}

void ItemDlg::slotSelectionChanged(QListViewItem *it)
{
  int i = lItem.findRef((KplCheckListItem *) it);
  up->setEnabled(i > 0);
  down->setEnabled(i < ((int) m->itd.count() - 1));
}

void ItemDlg::slotMoveUp()
{
  moveItem(true);
}

void ItemDlg::slotMoveDown()
{
  moveItem(false);
}
