/***************************************************************************
                          kplview.cpp  -  description                              
                             -------------------                                         

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Apr 24 15:14:00 MEST 1999
                                           
    copyright            : (C) 2000 by Werner Stille                         
    email                : stille@uni-freiburg.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <qscrollbar.h>
#include <qpaintdevicemetrics.h>
#include <qpixmap.h>
#include "kplview.h"
#include "kpldoc.h"
#include "kpl.h"
#include "kplwidget.h"
#include "kplitem.h"

KplView::KplView(KplApp *parent, double f) :
 QWidget(parent)
{
  doc = parent->getDocument();
  fZoom = f;
  fxn = fyn = 1.0;
  page = new KplWidget(this);
  sHScrollPos = sVScrollPos = 0;
  horScrollBar = new QScrollBar(QScrollBar::Horizontal, this);
  vertScrollBar = new QScrollBar(QScrollBar::Vertical, this);
  connect(horScrollBar, SIGNAL(valueChanged(int)),
          SLOT(slotHorScroll(int)));
  connect(vertScrollBar, SIGNAL(valueChanged(int)),
          SLOT(slotVertScroll(int)));
}

KplView::~KplView()
{
}

void KplView::print(QPaintDevice *printer)
{
  plInit(printer);
  doDraw();
  end();
}

void KplView::layout()
{
  bool hScroll = false, vScroll = false;

  QPaintDeviceMetrics pdm(this);
  xPict = qRound(10.0 * fZoom * doc->aut.xf * pdm.width() / pdm.widthMM());
  yPict = qRound(10.0 * fZoom * doc->aut.yf * pdm.height() / pdm.heightMM());
  scrx = width();
  scry = height();
  if (xPict > scrx) {
    hScroll = true;
    scry -= SCROLLBAR_SIZE;
  }
  if (yPict > scry) {
    vScroll = true;
    scrx -= SCROLLBAR_SIZE;
    if (!hScroll)
      if (xPict > scrx) {
        hScroll = true;
        scry -= SCROLLBAR_SIZE;
      }
  }
  if (hScroll) {
    horScrollBar->setGeometry(0, scry, scrx, SCROLLBAR_SIZE);
    horScrollBar->setRange(0, xPict - scrx);
    horScrollBar->setSteps(10, scrx);
    horScrollBar->setValue(sHScrollPos);
    horScrollBar->show();
  } else {
    horScrollBar->hide();
    sHScrollPos = 0;
  }
  if (vScroll) {
    vertScrollBar->setGeometry(scrx, 0, SCROLLBAR_SIZE, scry);
    vertScrollBar->setRange(0, yPict - scry);
    vertScrollBar->setSteps(10, scry);
    vertScrollBar->setValue(yPict - scry - sVScrollPos);
    vertScrollBar->show();
  } else {
    vertScrollBar->hide();
    sVScrollPos = 0;
  }
  page->resize(scrx, scry);
}

void KplView::doDraw()
{
  for (KplItem *item = doc->itd.first(); item; item = doc->itd.next())
    item->draw(this);
}

void KplView::eraseWidget()
{
  doPaint(false);
}

void KplView::doPaint(bool paint)
{
  page->setCursor(waitCursor);
  page->pm->resize(page->size());
  plInit(page->pm, sHScrollPos, sVScrollPos);
  if (paint) {
    factor = fZoom;
    format(doc->aut.xf, doc->aut.yf);
    doDraw();
  }
  end();
  bitBlt(page, 0, 0, page->pm);
  page->setCursor(crossCursor);
}

void KplView::slotScrollLeft()
{
  horScrollBar->subtractLine();
}

void KplView::slotScrollRight()
{
  horScrollBar->addLine();
}

void KplView::slotScrollUp()
{
  vertScrollBar->subtractLine();
}

void KplView::slotScrollDown()
{
  vertScrollBar->addLine();
}

void KplView::slotScrollPageUp()
{
  vertScrollBar->subtractPage();
}

void KplView::slotScrollPageDown()
{
  vertScrollBar->addPage();
}

void KplView::slotScrollPageLeft()
{
  horScrollBar->subtractPage();
}

void KplView::slotScrollPageRight()
{
  horScrollBar->addPage();
}

void KplView::slotScrollHomeLeft()
{
  horScrollBar->setValue(0);
}

void KplView::slotScrollHomeDown()
{
  vertScrollBar->setValue(vertScrollBar->maxValue());
}

void KplView::slotScrollEndRight()
{
  horScrollBar->setValue(horScrollBar->maxValue());
}

void KplView::slotScrollEndUp()
{
  vertScrollBar->setValue(0);
}

void KplView::paintWidget()
{
  doPaint(true);
}

void KplView::resizeEvent(QResizeEvent *)
{
  layout();
  doPaint(((KplApp *) parent())->plot);
}

void KplView::slotHorScroll(int pos)
{
  sHScrollPos = pos;
  paintWidget();
}

void KplView::slotVertScroll(int pos)
{
  sVScrollPos = vertScrollBar->maxValue() - pos;
  paintWidget();
}
