/***************************************************************************
                          lmfit.h  -  description                              
                             -------------------                                         
    begin                : Sat May 22 1999                                           
    copyright            : (C) 2000 by Werner Stille                         
    email                : stille@uni-freiburg.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef LMDIF_H
#define LMDIF_H

#include <qobject.h>

/**
  * Parameter fit class. Provides methods for nonlinear Levenberg-Marquardt
  * parameter fits and general linear least square parameter fits.
  * Uses code from public domain Fortran version of Argonne National Laboratories MINPACK,
  * C translation by Steve Moshier. Modifications by Werner Stille.
  */
class LMFit : public QObject {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   */
	LMFit(QObject *parent);
  /** Destructor. */
  ~LMFit();
  /** Performs general linear least square parameter fit. */
  bool linFit(int nd, int np, double px[], double fvec[], double a[],
              int ipvt[], double qtf[], double wa1[], double wa2[],
              double wa3[], double wa4[]);
  /**
   * Calculates weighed function difference vector.
   * @param nd number of data points.
   * @param np number of parameters to fit.
   * @param p array of parameters to fit.
   * @param fvec array of weighed function value differences (output).
   * @param iflag setting iflag < 0 by fcn causes termination of
   *        lmdif (input and output).
   */
  void fcn(int nd, int np, double p[], double fvec[], int *iflag);
  /** Performs nonlinear Levenberg-Marquardt parameter fit. */
  void lmdif(int nd, int np, double px[], double fvec[], double ftol,
             double ptol, double gtol, int maxfev, double epsfcn,
             double diag[], int mode, double factor, int nprint,
             int *info, int *nfev, double fjac[], int ldfjac,
             int ipvt[], double qtf[], double wa1[], double wa2[],
             double wa3[], double wa4[]);
  void lmpar(int n, double r[], int ldr, int ipvt[], double diag[],
             double qtb[], double delta, double *par, double x[],
             double sdiag[], double wa1[], double wa2[]);
  void fdjac2(int m, int n, double x[], double fvec[], double fjac[],
              int ldfjac, int *iflag, double epsfcn, double wa[]);
  void qrsolv(int n, double r[], int ldr, int ipvt[], double diag[],
              double qtb[], double x[], double sdiag[], double wa[]);
  void qrfac(int m, int n, double a[], int lda, int pivot, int ipvt[],
             int lipvt, double rdiag[], double acnorm[], double wa[]);
  double enorm(int n, double x[]);

protected:
  bool *bFit, userBreak;
  double *xLM, *yLM, *sig, *p;
  double (*fLM)(double, double *);

  friend class FitDlg;

signals:
  /**
    * Emitted when a message should be displayed.
    * @param msg message string
    */
  void updateMessage(const char *msg);
};

#endif
