/***************************************************************************
                          psgraph.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Wed Sep 08 1999
    copyright            : (C) 2000 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef PSGRAPH_H
#define PSGRAPH_H

#include <stdio.h>
#include "kplgraph.h"

/**
  * PostScript graphics class. Provides high level plotting functionality.
  * @author Werner Stille
  */

class PSGraph : public KplGraph  {

public:
  /** Constructor */
	PSGraph();
  /** Destructor */
	virtual ~PSGraph();
  /**
   * Initializes PostScript graphics.
   * @param f pointer to stream opened for write access.
   * @param xf width of bounding box.
   * @param yf height of bounding box.
   */
  void psInit(FILE *f, double xf, double yf);
  virtual void format(double x, double y);
  virtual void setColFrame(long icol1);
  virtual void setColGrid(long icol2);
  virtual void setColData(long icol3);
  virtual void setCol(long icol1, long icol2, long icol3);
  virtual double setRelSize(double r);
  virtual void Window(double x0, double xl, double y0, double yl);
  virtual void scale(double xl, double xr, double yu, double yo);
  virtual void setDig(int nx, int ny);
  virtual void frame();
  virtual void raster(double xtic, double ytic, int intx, int inty, int mode);
  virtual void textcm(char *str, double x, double y, int iorg);
  virtual void letH(char *str);
  virtual void letX(char* str, int iex);
  virtual void letY(char* str, int iex);
  virtual void setSymbol(int is);
  virtual void plArray(double *x, double *y, double fx, double fy, int n);
  virtual void plError(double *x, double *y, double *e, double fx, double fy, int n);
  virtual void plFun(double xfmin, double xfmax, double dx, double fx, double fy,
                     double (*fkt)(double, double *), double *p);
  virtual void plParFun(double tmin, double tmax, double dt, double fx, double fy,
                        double (*fktx)(double, double *), double *px,
                        double (*fkty)(double, double *), double *py);
  /** Ends PostScript graphic and closes output stream. */
  void psExit();
  virtual void cm2r(double xcm, double ycm, double fxn, double fyn,
                    double *xr, double *yr);

protected:
  void psStr(char *str);
  void text(char *str, double x, double y, int iorg);
  double x0, xl, y0, yl, xmin, xmax, ymin, ymax;
  FILE *fps;
};

#endif
